/* ##########################################################################

    Copyright 2004-2009:
    ANTONIO JAVIER BARRAGAN, antonio.barragan@diesia.uhu.es
    http://www.uhu.es/antonio.barragan
    JOSE MANUEL ANDUJAR, andujar@diesia.uhu.es

    DPTO. DE ING. ELECTRONICA, DE SISTEMAS INFORMATICOS Y AUTOMATICA
    E.P.S. LA RABIDA, UNIVERSITY OF HUELVA (SPAIN)

    For more information, please contact with authors.

########################################################################## */
/*
    This software is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "fuzzy_io.h"

#ifndef MATLAB_MEX_FILE
    // TYPE_MF
    std::ostream &operator<<(std::ostream &F, TYPE_MF &T)
    {
        F<<MF_NAMES[T];
        return F;
    }
    std::istream &operator>>(std::istream &F, TYPE_MF &T)
    {
	int iT;
        char type[MAX_SIZE_TYPE_NAME];
        F>>type;
        for (iT=(int)NOMF;iT<=(int)ZMF;iT++)
            if (!strcmpi(MF_NAMES[(TYPE_MF)iT],type))
                break;
        if (iT>(int)ZMF)
        {
        	std::cout<<E_MFType<<std::endl;
            exit(1);
        }
        return F;
    }

    // Membership
    std::ostream &operator<<(std::ostream &F, Membership &P)
    {
        F.precision(PRECISION);
        TYPE_MF T=P.type();
        F<<T<<"\t";
        for (int i=0;i<P.n_param();i++)
            F<<P.read(i)<<"\t";
        return F;
    }

    std::istream &operator>>(std::istream &F, Membership &P)
    {
        double d=0;
        if (P.type()==NOMF)
        {
            TYPE_MF t;
            F>>t;
            if(!P.type(t))
                std::cout<<E_MFTypeDef<<std::endl;;
        }
        for (int i=0;i<P.n_param();i++)
        {
            F>>d;
            if(!P.edit(i,d))
            {
            	std::cout<<E_ChangingParameter<<std::endl;
                exit(1);
            }
        }
        return F;
    }

    // Rule
    std::ostream &operator<<(std::ostream &F, Rule &R)
    {
        F.precision(PRECISION);
        int i;
        for (i=0;i<R.n_inputs();i++)
            F<<*R.read_function(i);
        F<<"\n";
        for (i=0;i<=R.n_inputs();i++)
            F<<R.read_TSK(i)<<"\t";
        F<<"\n";
        return F;
    }

    std::istream &operator>>(std::istream &F, Rule &R)
    {
        int i;
        double temp;
        TYPE_MF t;
        Membership *P;
        for (i=0;i<R.n_inputs();i++)
        {
            F>>t;
            if (!R.change_function(t,i))
                exit(1);
            P=createMF(t);
            F>>*P;
            if(!R.write_function(*P,i))
            {
            	std::cout<<E_WritingRule<<std::endl;
                exit(1);
            }
            delete P;
        }
        for (i=0;i<=R.n_inputs();i++)
        {
            F>>temp;
            if(!R.write_TSK(temp,i))
            {
            	std::cout<<E_WritingRule<<std::endl;
                exit(1);
            }
        }
        return F;
    }


    // System
    std::ostream &operator<<(std::ostream &F, System &S)
    {
        int i,j,n,m,in,out;
        int *rules=new int[S.outputs()];
        m=S.outputs();
		in=S.inputs();
        for (i=0;i<m;i++)
            *(rules+i)=S.rules(i);
        Rule R;
        F.precision(PRECISION);
        F<<n<<"\t"<<m<<"\n";
        for (j=0;j<m;j++)
            F<<S.rules(j)<<"\t";
        F<<"\n\n";
        for (j=0;j<m;j++)
        {
            for (i=0;i<rules[j];i++)
            {
                S.read_Rule(R,i,j);
                F<<R<<"\n";
            }
        }
        delete [] rules;
		F<<"\n";
		for (i=0;i<in;i++)
			F<<S.in_min(i)<<" "<<S.in_max(i)<<" ";
		F<<"\n";
		for (j=0;j<m;j++)
			F<<S.out_min(j)<<" "<<S.out_max(j)<<" ";
		F<<"\n";
        return F;
    }

    std::istream &operator>>(std::istream &F, System &S)
    {
        int temp,r,i,in,out;
		double *limits_min, *limits_max;
        F>>in;
        F>>out;
        int *rules=new int[out];
        for (i=0;i<out;i++)
            F>>*(rules+i);
        S.initialize(in,out,rules);
        Rule R(in);
        for (i=0;i<out;i++)
        {
            for (r=0;r<rules[i];r++)
            {
                F>>R;
                if(!S.write_Rule(R,r,i))
                    std::cout<<E_RulesExceded<<std::endl;
            }
        }
        delete [] rules;
		limits_min = new double[in];
		limits_max = new double[in];
		for (i=0;i<in;i++)
		{
			F>>limits_min[i];
			F>>limits_max[i];
		}
		S.in_min(limits_min);
		S.in_max(limits_max);
		delete [] limits_min;
		delete [] limits_max;
		limits_min = new double[out];
		limits_max = new double[out];
		for (i=0;i<out;i++)
		{
			F>>limits_min[i];
			F>>limits_max[i];
		}
		S.out_min(limits_min);
		S.out_max(limits_max);
		delete [] limits_min;
		delete [] limits_max;
        return F;
    }

    // Jacobian
    std::ostream &operator<<(std::ostream &F, Jacobian &J)
    {
        int i,q;
        F.precision(PRECISION);
        for (i=0;i<J.order();i++)
        {
            for (q=0;q<J.order();q++)
                    F<<J.read_J(i,q)<<"\t\t";
            F<<"\n";
        }
        return F;
    }

    std::istream &operator>>(std::istream &F, Jacobian &J)
    {
        int i,q;
        double temp;
        for (i=0;i<J.order();i++)
        {
            for (q=0;q<J.order();q++)
            {
                    F>>temp;
                    J.write_J(temp,i,q);
            }
        }
        return F;
    }
#endif
