/* ##########################################################################

    Copyright 2004-2009:
    ANTONIO JAVIER BARRAGAN, antonio.barragan@diesia.uhu.es
    http://www.uhu.es/antonio.barragan
    JOSE MANUEL ANDUJAR, andujar@diesia.uhu.es

    DPTO. DE ING. ELECTRONICA, DE SISTEMAS INFORMATICOS Y AUTOMATICA
    E.P.S. LA RABIDA, UNIVERSITY OF HUELVA (SPAIN)

    For more information, please contact with authors.

########################################################################## */

/*
    This software is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "system.h"

int System::rules(int output) const
{
    if (output>=out)
    {
        #ifndef MATLAB_MEX_FILE
            std::cout<<E_OutputExceded<<std::endl;
        #endif
        #ifdef MATLAB_MEX_FILE
            mexPrintf("%s\n",E_OutputExceded);
        #endif
        return 0;
    }
    return N[output];
}

int System::inputs(void) const {return in;}

int System::outputs(void) const {return out;}

double System::in_min(int input) const
{
    if (input>=in)
    {
        #ifndef MATLAB_MEX_FILE
            std::cout<<E_InputExceded<<std::endl;
        #endif
        #ifdef MATLAB_MEX_FILE
            mexPrintf("%s\n",E_InputExceded);
        #endif
        return MIN_DEFAULT_LIMIT;
    }
    return min_in[input];
}

double System::in_max(int input) const
{
    if (input>=in)
    {
        #ifndef MATLAB_MEX_FILE
            std::cout<<E_InputExceded<<std::endl;
        #endif
        #ifdef MATLAB_MEX_FILE
            mexPrintf("%s\n",E_InputExceded);
        #endif
        return MAX_DEFAULT_LIMIT;
    }
    return max_in[input];
}

double System::out_min(int output) const
{
    if (output>=out)
    {
        #ifndef MATLAB_MEX_FILE
            std::cout<<E_OutputExceded<<std::endl;
        #endif
        #ifdef MATLAB_MEX_FILE
            mexPrintf("%s\n",E_OutputExceded);
        #endif
        return MIN_DEFAULT_LIMIT;
    }
    return min_out[output];
}

double System::out_max(int output) const
{
    if (output>=out)
    {
        #ifndef MATLAB_MEX_FILE
            std::cout<<E_OutputExceded<<std::endl;
        #endif
        #ifdef MATLAB_MEX_FILE
            mexPrintf("%s\n",E_OutputExceded);
        #endif
        return MAX_DEFAULT_LIMIT;
    }
    return max_out[output];
}

void System::in_min(double *limits)
{
	for (int i=0;i<in;i++)
		min_in[i]=limits[i];
}

void System::in_max(double *limits)
{
	for (int i=0;i<in;i++)
		max_in[i]=limits[i];
}

void System::out_min(double *limits)
{
	for (int j=0;j<out;j++)
		min_out[j]=limits[j];
}

void System::out_max(double *limits)
{
	for (int j=0;j<out;j++)
		max_out[j]=limits[j];
}

int System::test(void) const
{
    int j;
    if (N==NULL || R==NULL || out==0 || in==0)
        return 0;
    else
    {
        for (j=0;j<out;j++)
        {
            if (R[j]==NULL)
                return 0;
            if (R[j]->test()==0)
                return 0;
        }
    }
    return 1;
}

int System::write_Rule(const Rule &R, int r, int output)
{
    if (output>=out || r>=N[output])
        return 0;
    this->R[output][r]=R;
    return 1;
}

int System::read_Rule(Rule &R, int r, int output) const
{
    if (output>=out || r>=N[output])
        return 0;
    R=this->R[output][r];
    return 1;
}

Rule * System::read_Rule(int r, int output)
{
    if (output>=out)
    {
        #ifndef MATLAB_MEX_FILE
            std::cout<<E_OutputExceded<<std::endl;
        #endif
        #ifdef MATLAB_MEX_FILE
            mexPrintf("%s\n",E_OutputExceded);
        #endif
        return NULL;
    }
    if (r>=N[output])
    {
        #ifndef MATLAB_MEX_FILE
            std::cout<<E_RulesExceded<<std::endl;
        #endif
        #ifdef MATLAB_MEX_FILE
            mexPrintf("%s\n",E_RulesExceded);
        #endif
        return NULL;
    }
    return &R[output][r];
}

void System::initialize(int in, int out, int *N)
{
    int r,i,j;
	if (in==0 || out ==0)
		return;
	if ((this->N)!=NULL)
		delete [] (this->N);
	this->N=new int[out];
	if (R!=NULL)
	{
		for (j=0;j<(this->out);j++)
			if (R[j]!=NULL)
				delete [] R[j];
		delete [] R;
	}
	if (min_in!=NULL)
		delete [] min_in;
	if (min_out!=NULL)
		delete [] min_out;
	if (max_in!=NULL)
		delete [] max_in;
	if (max_out!=NULL)
		delete [] max_out;
    this->in=in;
    this->out=out;
    R=new Rule*[out];
    min_in=new double [in];
	max_in=new double [in];
	min_out=new double [out];
	max_out=new double [out];
	for (i=0;i<in;i++)
	{
		min_in[i]=MIN_DEFAULT_LIMIT;
		max_in[i]=MAX_DEFAULT_LIMIT;
	}
    for (j=0;j<out;j++)
    {
		min_out[j]=MIN_DEFAULT_LIMIT;
		max_out[j]=MAX_DEFAULT_LIMIT;
        this->N[j]=N[j];
        R[j]=new Rule[N[j]];
        for (r=0;r<N[j];r++)
            R[j][r].initialize(in);
    }
}

System & System::operator=(const System &S)
{
    int r,j;
    if (this!=&S)
    {
        if (N!=NULL)
            delete [] N;
        N=new int[S.outputs()];
        if (R!=NULL)
        {
            for (j=0;j<out;j++)
                if (R[j]!=NULL)
                    delete [] R[j];
            delete [] R;
        }
		if (min_in!=NULL)
			delete [] min_in;
		if (min_out!=NULL)
			delete [] min_out;
		if (max_in!=NULL)
			delete [] max_in;
		if (max_out!=NULL)
			delete [] max_out;

        in=S.inputs();
        out=S.outputs();
        min_in=new double [in];
        max_in=new double [in];
        min_out=new double [out];
        max_out=new double [out];
        R=new Rule*[out];
		for (j=0;j<in;j++)
		{
			min_in[j]=S.in_min(j);
			max_in[j]=S.in_max(j);
		}
        for (j=0;j<out;j++)
        {
			min_out[j]=S.out_min(j);
			max_out[j]=S.out_max(j);
            N[j]=S.rules(j);
            R[j]=new Rule[N[j]];
            for (r=0;r<N[j];r++)
                R[j][r]=S.R[j][r];
        }
    }
    return *this;
}

System::System()
{
    in=0;
    out=0;
    N=NULL;
    R=NULL;
	min_in=NULL;
	max_in=NULL;
	min_out=NULL;
	max_out=NULL;
}

System::System(const System &S)
{
    int r,i,j;
    in=S.inputs();
    out=S.outputs();
    min_in=new double [in];
	max_in=new double [in];
	min_out=new double [out];
	max_out=new double [out];
	N=new int[out];
    R=new Rule*[out];
	for (i=0;i<in;i++)
	{
        min_in[i]=S.in_min(i);
        max_in[i]=S.in_max(i);
	}
    for (j=0;j<out;j++)
    {
		min_out[j]=S.out_min(j);
		max_out[j]=S.out_max(j);
        N[j]=S.rules(j);
        R[j]=new Rule[N[j]];
        for (r=0;r<N[j];r++)
            R[j][r]=S.R[j][r];
    }
}

System::System(int in, int out, int *N)
{
    int r,j,i;
    this->in=in;
    this->out=out;
    min_in=new double [in];
	max_in=new double [in];
	min_out=new double [out];
	max_out=new double [out];
    this->N=new int[out];
	for (i=0;i<in;i++)
	{
		min_in[i]=MIN_DEFAULT_LIMIT;
		max_in[i]=MAX_DEFAULT_LIMIT;
	}
    R=new Rule*[out];
    for (j=0;j<out;j++)
    {
		min_out[j]=MIN_DEFAULT_LIMIT;
		max_out[j]=MAX_DEFAULT_LIMIT;
        this->N[j]=N[j];
        R[j]=new Rule[N[j]];
        for (r=0;r<N[j];r++)
            R[j][r].initialize(in);
    }
}

System::~System()
{
    int j;
	if (min_in!=NULL)
	{
		delete [] min_in;
		min_in=NULL;
	}
	if (max_in!=NULL)
	{
		delete [] max_in;
		max_in=NULL;
	}
	if (min_out!=NULL)
	{
		delete [] min_out;
		min_out=NULL;
	}
	if (max_out!=NULL)
	{
		delete [] max_out;
		max_out=NULL;
	}
    if (N!=NULL)
    {
        delete [] N;
        N=NULL;
    }
    if (R!=NULL)
    {
        for (j=0;j<out;j++)
        {
            if (R[j]!=NULL)
                delete [] R[j];
            R[j]=NULL;
        }
        delete [] R;
        R=NULL;
    }
}
