/* ##########################################################################

    Copyright 2004-2009:
    ANTONIO JAVIER BARRAGAN, antonio.barragan@diesia.uhu.es
    http://www.uhu.es/antonio.barragan
    JOSE MANUEL ANDUJAR, andujar@diesia.uhu.es

    DPTO. DE ING. ELECTRONICA, DE SISTEMAS INFORMATICOS Y AUTOMATICA
    E.P.S. LA RABIDA, UNIVERSITY OF HUELVA (SPAIN)

    For more information, please contact with authors.

########################################################################## */

/*
    This software is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _UTILITIES_H_
#define _UTILITIES_H_

#include <stdlib.h>
#include <iostream>
#include <strings.h>
#include "membership.h"
#include "rule.h"
#include "system.h"
#include "messages.h"

#ifndef strcmpi
    #define strcmpi strcasecmp // Make the change to compile in GNU/Linux
#endif

// Declarations ################################################################

int TXT2System(const char *file, System &S);

int System2TXT(System &S, const char *file); // Write a TXT file with the system 'S'

int Evaluate(const double * const x, double *output, System &S); // Evaluate 'S' in 'x'. Return -1 if an overflow occured or 1 if all went well

int Evaluate(const double * const x, double *dX, System &S, System &C); // Evaluate the closed loop of the plant 'S' with the controller 'C' in 'x'. Return -1 if an overflow occured, 0 if the models are not compatible or 1 if all went well

int linearization(System &S, const double *const x, const double *const u, double **A, double **B, double *F=NULL); // Computes the linearization of the system 'S' in 'x'. Return 0 if failed and 1 if all went well

int linearaprox(System &S, const double *const x, const double *const u, double **A, double **B, double h=0.001, double *F=NULL); // Computes the linearization of the system 'S' in 'x'. Return 0 if failed and 1 if all went well

int printSystem(const char *file, const System &S, char *inputs[]=NULL, char *outputs[]=NULL, int accuracy=10); // Write the system 'S' in its linguistic form: "IF x1 is A111 and x2 is ... THEN dx1 is ..."

int NumbersOfAntecedentsData(System &S); // Gets the number of data stored in the antecedents of the fuzzy model 'S' (all parameters of membership functions)

int NumbersOfAntecedentsData(System &S, int out, int rule); // Gets the number of data stored in the antecedents for the rule 'rule' for the output 'out' in the fuzzy model 'S' (all parameters of membership functions)

int NumbersOfConsequentsData(System &S); // Gets the number of data stored in the consequents of the fuzzy model 'S'

int NumbersOfConsequentsData(System &S, int out, int rule); // Gets the number of data stored in the consequents for the rule 'rule' for the output 'out' in the fuzzy model 'S'

int getAntecedents(System &S, double * parameters); // Get all antecedents data. Return 0 if failed and the number of parameters if all went well

int getAntecedents(System &S, double * parameters, int out, int rule); // Get all antecedents data for rule 'rule' of output 'out'. Return 0 if failed and the number of parameters if all went well

int getConsequents(System &S, double * parameters); // Get all consequent data. Return 0 if failed and the number of parameters if all went well

int getConsequents(System &S, double * parameters, int out, int rule); // Get all antecedents data for rule 'rule' of output 'out'. Return 0 if failed and the number of parameters if all went well

int setAntecedents(System &S, const double * const parameters); // Set all antecedents data from 'parameters'. Return 0 if failed and 1 if all went well

int setAntecedents(System &S, const double * const parameters, int out, int rule); // Set all antecedents data for rule 'rule' of output 'out' from 'parameters'. Return 0 if failed and 1 if all went well

int setConsequents(System &S, const double * const parameters); // Set all consequents data from 'parameters'. Return 0 if failed and 1 if all went well

int setConsequents(System &S, const double * const parameters, int out, int rule); // Set all consequents data for rule 'rule' of output 'out' from 'parameters'. Return 0 if failed and 1 if all went well

int activation(System &S, int output, int rule, double *point, double* W); // Get the matching degree of the indicated rule and point. Return 0 if all went well

int activation(System &S, int output, int rule, double *point, double* W, double *dW); // Get the matching degree of the indicated rule and point and its derivatives. Return 0 if all went well

int subSystem(System &S,System &ParteS, int nrules, int *outputs, int *rules); // Get a subsystem from 'S'. Return 0 if all went well

#endif
